import { Extension, TransactionSpec, EditorState } from '@codemirror/state';
import { VirtualTypeScriptEnvironment } from '@typescript/vfs';

declare type FileMap = Record<string, string>;
/**
 * A representation of a Typescript project. Only supports single-file projects currently.
 */
declare class TypescriptProject {
    private fs;
    /**
     * Since this module is lazily initialized, this serves as a way to throttle multiple consecutive `init` requests.
     * We want to avoid initializing `tsserver` multiple times.
     *
     * After construction, it stays in the `procrastinating` state until someone requests something of it.
     * Once that happens, it goes through the `initializing` & `ready` states.
     */
    private state;
    /** When initialization starts, the promise it returns is stored here so that future `init` requests can be throttled */
    private initPromise?;
    private tsserver?;
    constructor(entrypointFileContent: string);
    get entrypoint(): string;
    private init;
    injectTypes(types: FileMap): Promise<void>;
    env(): Promise<VirtualTypeScriptEnvironment>;
    lang(): Promise<VirtualTypeScriptEnvironment['languageService']>;
    destroy(): void;
}

declare const tsTheme: Extension;

declare function injectTypes(types: FileMap): TransactionSpec;
/**
 * A TransactionSpec that can be dispatched to force re-calculation of lint diagnostics
 */
declare function setDiagnostics(state: EditorState): Promise<TransactionSpec>;
declare function typescript(): Extension;

export { FileMap, TypescriptProject, injectTypes, setDiagnostics, tsTheme, typescript };
